@@ -15,7 +15,8 @@ use Module::Build::Base;
 
 use vars qw($VERSION @ISA);
 @ISA = qw(Module::Build::Base);
-$VERSION = '0.25';
+$VERSION = '0.26';
+$VERSION = eval $VERSION;
 
 # Okay, this is the brute-force method of finding out what kind of
 # platform we're on.  I don't know of a systematic way.  These values
@@ -118,7 +119,7 @@ the "./" notation, you can do this:
 =head1 DESCRIPTION
 
 C<Module::Build> is a system for building, testing, and installing
-Perl modules.  It is meant to be a replacement for
+Perl modules.  It is meant to be an alternative to
 C<ExtUtils::MakeMaker>.  Developers may alter the behavior of the
 module through subclassing in a much more straightforward way than
 with C<MakeMaker>.  It also does not require a C<make> on your system
@@ -141,20 +142,21 @@ C<Module::Build> for its installation process, do the following:
 
 This illustrates initial configuration and the running of three
 'actions'.  In this case the actions run are 'build' (the default
-action), 'test', and 'install'.  Actions defined so far include:
+action), 'test', and 'install'.  Other actions defined so far include:
 
-  build                          fakeinstall   
-  clean                          help          
-  code                           install       
-  diff                           manifest      
-  dist                           ppd           
-  distcheck                      ppmdist       
-  distclean                      realclean     
-  distdir                        skipcheck     
-  distmeta                       test          
-  distsign                       testdb        
-  disttest                       versioninstall
-  docs
+  build                          fakeinstall 
+  config_data                    help        
+  clean                          html        
+  code                           install     
+  diff                           manifest    
+  dist                           ppd         
+  distcheck                      ppmdist     
+  distclean                      realclean   
+  distdir                        skipcheck   
+  distmeta                       test        
+  distsign                       testcover   
+  disttest                       testdb      
+  docs                           versioninstall
 
 You can run the 'help' action for a complete list of actions.
 
@@ -198,7 +200,7 @@ the default 'test' action.  You could also add a method called
 C<ACTION_whatever>, and then you could perform the action C<Build
 whatever>.
 
-For information on providing backward compatibility with
+For information on providing compatibility with
 C<ExtUtils::MakeMaker>, see L<Module::Build::Compat> and
 L<http://www.makemaker.org/wiki/index.cgi?ModuleBuildConversionGuide>.
 
@@ -615,8 +617,121 @@ it gets the distribution's version.  It looks for the first line
 matching C<$package\s-\s(.+)>, and uses the captured text as the
 abstract.
 
+=item auto_features
+
+This parameter supports the setting of features (see
+L<feature($name)>) automatically based on a set of prerequisites.  For
+instance, for a module that could optionally use either MySQL or
+PostgreSQL databases, you might use C<auto_features> like this:
+
+  my $b = Module::Build->new
+    (
+     ... other stuff here...
+     auto_features =>
+       {
+        pg_support =>
+        {
+           description => "Interface with Postgres databases",
+           requires => q{ DBD::Pg >= 23.3 && DateTime::Format::Pg },
+        },
+        mysql_support =>
+        {
+           description => "Interface with MySQL databases",
+           requires => q{ DBD::mysql >= 17.9 && DateTime::Format::Pg },
+        },
+     );
+
+For each feature named, the prerequisite options will be checked, and
+if there are no failures, the feature will be enabled (set to C<1>).
+Otherwise the failures will be displayed to the user and the feature
+will be disabled (set to C<0>).
+
+=item get_options
+
+You can pass arbitrary command-line options to F<Build.PL> or F<Build>, and they will be
+stored in the Module::Build object and can be accessed via the C<args()>
+method. However, sometimes you want more flexibility out of your argument
+processing than this allows. In such cases, use the C<get_options> parameter
+to pass in a hash reference of argument specifications, and the list of
+arguments to F<Build.PL> or F<Build> will be processed according to those
+specifications before they're passed on to C<Module::Build>'s own argument
+processing.
+
+The supported option specification hash keys are:
+
+=over 4
+
+=item type
+
+The type of option. The types are those supported by Getopt::Long; consult
+its documentation for a complete list. Typical types are C<=s> for strings,
+C<+> for additive options, and C<!> for negatable options.  If the
+type is not specified, it will be considered a boolean, i.e. no
+argument is taken and a value of 1 will be assigned when the option is
+encountered.
+
+=item store
+
+A reference to a scalar in which to store the value passed to the option.
+If not specified, the value will be stored under the option name in the
+hash returned by the C<args()> method.
+
+=item default
+
+A default value for the option. If no default value is specified and no option
+is passed, then the option key will not exist in the hash returned by
+C<args()>.
+
 =back
 
+You can combine references to your own variables or subroutines with
+unreferenced specifications, for which the result will also be stored in the
+has returned by C<args()>. For example:
+
+ my $loud = 0;
+ my $build = Module::Build->new(
+     module_name => 'Spangly',
+     get_options => {
+                      loud =>     { store => \$loud },
+                      dbd  =>     { type  => '=s'   },
+                      quantity => { type  => '+'    },
+                    }
+ );
+
+ print STDERR "HEY, ARE YOU LISTENING??\n" if $loud;
+ print "We'll use the ", $build->args('dbd'), " DBI driver\n";
+ print "Are you sure you want that many?\n"
+   if $build->args('quantity') > 2;
+
+The arguments for such a specification can be called like so:
+
+ % perl Build.PL --loud --dbd=DBD::pg --quantity --quantity --quantity
+
+B<WARNING:> Any option specifications that conflict with Module::Build's own
+options (defined by its properties) will throw an exception.
+
+Consult the Getopt::Long documentation for details on its usage.
+
+=back
+
+=item args()
+
+  my $args_href = $build->args;
+  my %args = $build->args;
+  my $arg_value = $build->args($key);
+  $build->args($key, $value);
+
+This method is the preferred interface for retreiving the arguments passed via
+command-line options to F<Build.PL> or F<Build>, minus the Module-Build
+specific options.
+
+When called in in a scalar context with no arguments, this method returns a
+reference to the hash storing all of the arguments; in an array context, it
+returns the hash itself. When passed a single argument, it returns the value
+stored in the args hash for that option key. When called with two arguments,
+the second argument is assigned to the args hash under the key passed as the
+first argument.
+
 =item subclass()
 
 This creates a new C<Module::Build> subclass on the fly, as described
@@ -637,13 +752,20 @@ when the C<realclean> action is performed.
 
 =item add_to_cleanup(@files)
 
-You may call C<< $self->add_to_cleanup(@files) >>
-to tell C<Module::Build> that certain files should be removed when the
-user performs the C<Build clean> action.  I decided to provide a
-dynamic method, rather than just use a static list of files, because these
-static lists can get difficult to manage.  I usually prefer to keep
-the responsibility for registering temporary files close to the code
-that creates them.
+You may call C<< $self->add_to_cleanup(@patterns) >> to tell
+C<Module::Build> that certain files should be removed when the user
+performs the C<Build clean> action.  The arguments to the method are
+patterns suitable for passing to Perl's C<glob()> function, specified
+in either Unix format or the current machine's native format.  It's
+usually convenient to use Unix format when you hard-code the filenames
+(e.g. in F<Build.PL>) and the native format when the names are
+programmatically generated (e.g. in a testing script).
+
+I decided to provide a dynamic method of the C<$build> object, rather
+than just use a static list of files named in the F<Build.PL>, because
+these static lists can get difficult to manage.  I usually prefer to
+keep the responsibility for registering temporary files close to the
+code that creates them.
 
 =item resume()
 
@@ -660,7 +782,7 @@ This method returns a reasonable faxsimile of the currently-executing
 C<Module::Build> object representing the current build.  You can use
 this object to query its C<notes()> method, inquire about installed
 modules, and so on.  This is a great way to share information between
-different parts of your building process.  For instance, you can ask
+different parts of your build process.  For instance, you can ask
 the user a question during C<perl Build.PL>, then use their answer
 during a regression test:
 
@@ -680,6 +802,10 @@ one in C<t/colortest.t>.  It is our goal to minimize these differences
 in future releases of Module::Build, so please report any anomalies
 you find.
 
+One important caveat: in its current implementation, C<current()> will
+B<NOT> work correctly if you have changed out of the directory that
+C<Module::Build> was invoked from.
+
 =item notes()
 
 =item notes($key)
@@ -697,6 +823,12 @@ With one argument, C<notes($key)> returns the value associated with
 the given key.  With two arguments, C<notes($key, $value)> sets the
 value associated with the given key to C<$value>.
 
+The lifetime of the C<notes> data is for "a build" - that is, the
+C<notes> hash is created when C<perl Build.PL> is run (or when the
+C<new()> method is run, if the Module::Build Perl API is being used
+instead of called from a shell), and lasts until C<perl Build.PL> is
+run again or the C<clean> action is run.
+
 =item config()
 
 Returns a hash reference containing the C<Config.pm> hash, including
@@ -887,6 +1019,16 @@ same thing as C<script_files()>.  C<scripts()> is deprecated, but it
 will stay around for several versions to give people time to
 transition.
 
+=item add_build_element($type)
+
+Adds a new type of entry to the build process.  Accepts a single
+string specifying its type-name.  There must also be a method defined
+to process things of that type, e.g. if you add a build element called
+C<'foo'>, then you must also define a method called
+C<process_foo_files()>.
+
+See also L<Module::Build::Compat/Adding new elements to the build process>.
+
 =item copy_if_modified(%parameters)
 
 Takes the file in the C<from> parameter and copies it to the file in
@@ -961,6 +1103,51 @@ Currently this checks whether the file has a line beginning with
 '=pod', '=head', or '=item', but the exact semantics may change in the
 future.
 
+=item feature($name)
+
+=item feature($name => $value)
+
+With a single argument, returns true if the given feature is set.
+With two arguments, sets the given feature to the given boolean value.
+In this context, a "feature" is any optional functionality of an
+installed module.  For instance, if you write a module that could
+optionally support a MySQL or PostgreSQL backend, you might create
+features called C<mysql_support> and C<postgres_support>, and set them
+to true/false depending on whether the user has the proper databases
+installed and configured.
+
+Features set in this way using the Module::Build object will be
+available for querying during the build/test process and after
+installation via the generated C<...::ConfigData> module, as 
+C<< ...::ConfigData->feature($name) >>.
+
+The C<feature()> and C<config_data()> methods represent
+Module::Build's main support for configuration of installed modules.
+See also L<SAVING CONFIGURATION INFORMATION>.
+
+=item config_data($name)
+
+=item config_data($name => $value)
+
+With a single argument, returns the value of the configuration
+variable C<$name>.  With two arguments, sets the given configuration
+variable to the given value.  The value may be any perl scalar that's
+serializable with C<Data::Dumper>.  For instance, if you write a
+module that can use a MySQL or PostgreSQL backend, you might create
+configuration variables called C<mysql_connect> and
+C<postgres_connect>, and set each to an array of connection parameters
+for C<< DBI->connect() >>.
+
+Configuration values set in this way using the Module::Build object
+will be available for querying during the build/test process and after
+installation via the generated C<...::ConfigData> module, as 
+C<< ...::ConfigData->config($name) >>.
+
+The C<feature()> and C<config_data()> methods represent
+Module::Build's main support for configuration of installed modules.
+See also L<SAVING CONFIGURATION INFORMATION>.
+
+
 =back
 
 =head1 ACTIONS
@@ -1085,12 +1272,23 @@ or use a C<glob()>-style pattern:
 
  ./Build test --test_files 't/01-*.t'
 
+=item testcover
+
+Runs the C<test> action using C<Devel::Cover>, generating a
+code-coverage report showing which parts of the code were actually
+exercised during the tests.
 
 =item testdb
 
 This is a synonym for the 'test' action with the C<debugger=1>
 argument.
 
+=item testpod
+
+This checks all the files described in the C<docs> action and 
+produces C<Test::Harness>-style output. If you are a module author,
+this is useful to run before creating a new release.
+
 =item clean
 
 This action will clean up any files that the build process may have
@@ -1331,7 +1529,12 @@ pages belonging to the 'man1' category.
 
 =back
 
-=head3 installdirs
+Four other parameters let you control various aspects of how
+installation paths are determined:
+
+=over 4
+
+=item installdirs
 
 The default destinations for these installable things come from
 entries in your system's C<Config.pm>.  You can select from three
@@ -1372,7 +1575,7 @@ unfortunately there's no such thing as "installsitescript" or
 general location right.  In the future, if C<Config.pm> adds some more
 appropriate entries, we'll start using those.)
 
-=head3 install_path
+=item install_path
 
 Once the defaults have been set, you can override them.  You can set
 individual entries by using the C<install_path> parameter:
@@ -1390,7 +1593,7 @@ or this:
 
  Build install --install_path lib=/foo/lib --install_path arch=/foo/lib/arch
 
-=head3 install_base
+=item install_base
 
 You can also set the whole bunch of installation paths by supplying the
 C<install_base> parameter to point to a directory on your system.  For
@@ -1401,8 +1604,8 @@ system, you'll install as follows:
  arch    => /home/ken/lib/i386-linux
  script  => /home/ken/scripts
  bin     => /home/ken/bin
- libdoc  => /home/ken/man/man1
- bindoc  => /home/ken/man/man3
+ bindoc  => /home/ken/man/man1
+ libdoc  => /home/ken/man/man3
 
 Note that this is I<different> from how MakeMaker's C<PREFIX>
 parameter works.  C<PREFIX> tries to create a mini-replica of a
@@ -1415,7 +1618,7 @@ C<installdirs=site> layout.
 The exact layout under the directory you specify may vary by system -
 we try to do the "sensible" thing on each platform.
 
-=head3 destdir
+=item destdir
 
 If you want to install everything into a temporary directory first
 (for instance, if you want to create a directory tree that a package
@@ -1433,6 +1636,24 @@ This will effectively install to "/tmp/foo/$sitelib",
 C<File::Spec> to make the pathnames work correctly on whatever
 platform you're installing on.
 
+=back
+
+=head1 SAVING CONFIGURATION INFORMATION
+
+Module::Build provides a very convenient way to save configuration
+information that your installed modules (or your regression tests) can
+access.  If your Build process calls the C<feature()> or
+C<config_data()> methods, then a C<Foo::Bar::ConfigData> module will
+automatically be created for you, where C<Foo::Bar> is the
+C<module_name> parameter as passed to C<new()>.  This module provides
+access to the data saved by these methods, and a way to update the
+values.  There is also a utility script called C<config_data>
+distributed with Module::Build that provides a command-line interface
+to this same functionality.  See also the generated
+C<Foo::Bar::ConfigData> documentation, and the C<config_data>
+script's documentation, for more information.
+
+
 
 =head1 AUTOMATION
 
@@ -1496,7 +1717,11 @@ Module::Build directly:
   use lib qw(/nonstandard/library/path);
   use My::Builder;  # Or whatever you want to call it
   
-  my $m = My::Builder->new(module_name => 'Next::Big::Thing');
+  my $m = My::Builder->new
+    (module_name=> 'Next::Big::Thing',  # All the regular args...
+     license=> 'perl',
+     dist_author=> 'A N Other <me@here.net.au>',
+     requires=> {Carp => 0});
   $m->create_build_script;
 
 This is relatively straightforward, and is the best way to do things
@@ -1524,7 +1749,11 @@ creating a separate file for your module:
      },
     );
   
-  my $m = $class->new(module_name => 'Module::Build');
+  my $m = $class->new
+    (module_name=> 'Next::Big::Thing',  # All the regular args...
+     license=> 'perl',
+     dist_author=> 'A N Other <me@here.net.au>',
+     requires=> {Carp => 0});
   $m->create_build_script;
 
 Behind the scenes, this actually does create a C<.pm> file, since the
@@ -1570,12 +1799,12 @@ build/install process to do what they want.
 
 There are several architectural decisions in MakeMaker that make it
 very difficult to customize its behavior.  For instance, when using
-MakeMaker you do C<use MakeMaker>, but the object created in
+MakeMaker you do C<use ExtUtils::MakeMaker>, but the object created in
 C<WriteMakefile()> is actually blessed into a package name that's
 created on the fly, so you can't simply subclass
 C<ExtUtils::MakeMaker>.  There is a workaround C<MY> package that lets
 you override certain MakeMaker methods, but only certain explicitly
-predefined (by MakeMaker) methods can be overridden.  Also, the method
+preselected (by MakeMaker) methods can be overridden.  Also, the method
 of customization is very crude: you have to modify a string containing
 the Makefile text for the particular target.  Since these strings
 aren't documented, and I<can't> be documented (they take on different
@@ -1588,7 +1817,7 @@ perl.
 
 It is risky to make major changes to MakeMaker, since it does so many
 things, is so important, and generally works.  C<Module::Build> is an
-entirely seperate package so that I can work on it all I want, without
+entirely separate package so that I can work on it all I want, without
 worrying about backward compatibility.
 
 =item *
@@ -1597,7 +1826,8 @@ Finally, Perl is said to be a language for system administration.
 Could it really be the case that Perl isn't up to the task of building
 and installing software?  Even if that software is a bunch of stupid
 little C<.pm> files that just need to be copied from one place to
-another?  Are you getting riled up yet??
+another?  My sense was that we could design a system to accomplish
+this in a flexible, extensible, and friendly manner.  Or die trying.
 
 =back
 
